"""
NUVA SDK - Unified Vaccine Nomenclature

A Python SDK for working with the Unified Vaccine Nomenclature (NUVA),
designed to aggregate vaccination histories from both digital and physical sources.
"""

from .nuva import Nuva, Repository, NuvaRepositories, NuvaQueries

__version__ = "1.0.0"
__all__ = ["Nuva", "Repository", "NuvaRepositories", "NuvaQueries"]



from .protobuf.NuvaDatabase_pb2 import NuvaDatabase
from .queries import ValencesByVaccine, VaccinesByValence, VaccinesByDisease, ValencesByDisease, DiseasesByVaccine, DiseasesByValence
import requests
import json

class Repository:
    def __init__(self, data):
        self.data = data
        self.indexTable = {}

        for x in data:
            self.indexTable[x.id] = x
    
    def find(self, id):
        if id in self.indexTable:
            return self.indexTable[id]
        return None

    def all(self):
        return self.data

class NuvaRepositories:
    def __init__(self, db):
        self.vaccines = Repository(db.vaccines)
        self.valences = Repository(db.valences)
        self.diseases = Repository(db.diseases)

class NuvaQueries:
    def __init__(self, repositories):
        self.valences_by_vaccine = ValencesByVaccine(repositories)
        self.vaccine_by_valences = VaccinesByValence(repositories)
        self.vaccines_by_disease = VaccinesByDisease(repositories)
        self.valences_by_disease = ValencesByDisease(repositories)
        self.diseases_by_vaccine = DiseasesByVaccine(repositories)
        self.diseases_by_valence = DiseasesByValence(repositories)

class Nuva:
    def __init__(self, db):
        self.db = db
        self.repositories = NuvaRepositories(db)
        self.queries = NuvaQueries(self.repositories)

    @staticmethod
    def load(lang = 'en'):
        manifest = requests.get('https://cdn.nuva.fr/versions/last.json').json()
        nuva = requests.get('https://cdn.nuva.fr/proto/%s_%s.db' % (manifest['dump_hash'], lang))
        db = NuvaDatabase()
        db.ParseFromString(nuva.content)
        return Nuva(db)

    @staticmethod
    def load_from_file(path):
        db = NuvaDatabase()
        with open(path, 'rb') as f:
            db.ParseFromString(f.read())
        return Nuva(db)



class ValencesByVaccine:
    def __init__(self, repositories):
        self.valences_by_vaccine_id = {}
        for valence in repositories.valences.all():
            for vaccine_id in valence.vaccine_ids:
                self.valences_by_vaccine_id.setdefault(vaccine_id, []).append(valence)
    
    def call(self, vaccine):
        return self.valences_by_vaccine_id.get(vaccine.id, [])


class VaccinesByValence:
    def __init__(self, repositories):
        self.vaccines_by_valence = {}
        for vaccine in repositories.vaccines.all():
            for vaccine_id in vaccine.valence_ids:
                self.vaccines_by_valence.setdefault(vaccine_id, []).append(vaccine)
    
    def call(self, vaccine):
        return self.vaccines_by_valence.get(vaccine.id, [])

class VaccinesByDisease:
    def __init__(self, repositories):
        self.vaccines_by_disease_id = {}
        for valence in repositories.valences.all():
            for disease_id in valence.disease_ids:
                self.vaccines_by_disease_id.setdefault(disease_id, [])
                for vaccine_id in valence.vaccine_ids:
                    self.vaccines_by_disease_id[disease_id].append(repositories.vaccines.find(vaccine_id))

    def call(self, disease):
        return self.vaccines_by_disease_id.get(disease.id, [])

class ValencesByDisease:
    def __init__(self, repositories):
        self.valences_by_disease_id = {}
        for valence in repositories.valences.all():
            for disease_id in valence.disease_ids:
                self.valences_by_disease_id.setdefault(disease_id, []).append(valence)

    def call(self, disease):
        return self.valences_by_disease_id.get(disease.id, [])

class DiseasesByVaccine:
    def __init__(self, repositories):
        self.diseases_by_vaccine_id = {}
        for valence in repositories.valences.all():
            for vaccine_id in valence.vaccine_ids:
                for disease_id in valence.disease_ids:
                    self.diseases_by_vaccine_id.setdefault(vaccine_id, []).append(repositories.diseases.find(disease_id))

    def call(self, vaccine):
        return self.diseases_by_vaccine_id.get(vaccine.id, [])

class DiseasesByValence:
    def __init__(self, repositories):
        self.diseases_by_valence_id = {}
        for disease in repositories.diseases.all():
            for valence_id in disease.valence_ids:
                self.diseases_by_valence_id.setdefault(valence_id, []).append(disease)

    def call(self, valence):
        return self.diseases_by_valence_id.get(valence.id, [])



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: Disease.proto
# Protobuf Python Version: 5.28.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    2,
    '',
    'Disease.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
import Code_pb2 as Code__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rDisease.proto\x12\x04nuva\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\nCode.proto\"\xe9\x01\n\x07\x44isease\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04\x63ode\x18\x03 \x01(\r\x12\x19\n\x05\x63odes\x18\x04 \x03(\x0b\x32\n.nuva.Code\x12\x12\n\nvaccinable\x18\x05 \x01(\x08\x12\x12\n\nscreenable\x18\x06 \x01(\x08\x12\x13\n\x0bvalence_ids\x18\x07 \x03(\t\x12.\n\ncreated_at\x18\x10 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nupdated_at\x18\x11 \x01(\x0b\x32\x1a.google.protobuf.TimestampB!\n\x0f\x63om.syadem.nuvaP\x01\xaa\x02\x0bSyadem.Nuvab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Disease_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\017com.syadem.nuvaP\001\252\002\013Syadem.Nuva'
  _globals['_DISEASE']._serialized_start=69
  _globals['_DISEASE']._serialized_end=302
# @@protoc_insertion_point(module_scope)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="nuva-sdk",
    version="1.0.0",
    author="NUVA Team",
    author_email="contact@nuva.fr",
    description="Unified Vaccine Nomenclature (NUVA) Python SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nuva/nuva-libs",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Healthcare Industry",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.7",
    install_requires=[
        "protobuf>=3.20.0",
        "requests>=2.25.0",
    ],
    keywords="vaccine, vaccination, nomenclature, healthcare, immunization",
    project_urls={
        "Bug Reports": "https://github.com/nuva/nuva-libs/issues",
        "Source": "https://github.com/nuva/nuva-libs",
        "Documentation": "https://nuva.fr/docs",
    },
)
